"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useMenuItemTracking = exports.useMenuList = void 0;
var tslib_1 = require("tslib");
var react_1 = tslib_1.__importDefault(require("react"));
var react_native_1 = require("react-native");
var menuContext_1 = require("../context/menuContext");
var menuListContext_1 = require("../context/menuListContext");
// Track the radio items so we know what to clear selection from when selectRadio is called
// Purposefully left out of the hook because
// 1. RadioItems just keeps track of information - changing this array doesn't need to force rerender
// 2. Keeping them here means these consts are not recreated on every render, which would force rerendering of all children
var radioItems = [];
var addRadioItem = function (name) {
    radioItems.push(name);
};
var removeRadioItem = function (name) {
    radioItems.filter(function (item) { return item !== name; });
};
var platformsWithoutFocusOnDisabled = ['ios', 'macos'];
var handledKeys = ['Home', 'End'];
var handleFocusOnMouseLevae = react_native_1.Platform.OS === 'macos';
var useMenuList = function (_props) {
    var _a, _b;
    var context = (0, menuContext_1.useMenuContext)();
    // MenuList v2 needs to be able to be standalone, but this is not in scope for v1
    // Assuming that checked information will come from parent Menu
    var defaultChecked = context.defaultChecked, onCheckedChangeOriginal = context.onCheckedChange, checkedOriginal = context.checked, isSubmenu = context.isSubmenu, setOpen = context.setOpen, triggerRef = context.triggerRef;
    // Convert passed in array to map so that i's easier to look up checked state
    var checkedMap = react_1.default.useMemo(function () {
        var state = {};
        if (!checkedOriginal) {
            return state;
        }
        for (var _i = 0, checkedOriginal_1 = checkedOriginal; _i < checkedOriginal_1.length; _i++) {
            var key = checkedOriginal_1[_i];
            state[key] = true;
        }
        return state;
    }, [checkedOriginal]);
    var _c = react_1.default.useState(function () {
        if (checkedMap) {
            return checkedMap;
        }
        var initialChecked = defaultChecked !== null && defaultChecked !== void 0 ? defaultChecked : [];
        var state = {};
        for (var _i = 0, initialChecked_1 = initialChecked; _i < initialChecked_1.length; _i++) {
            var key = initialChecked_1[_i];
            state[key] = true;
        }
        return state;
    }), checkedInternal = _c[0], setCheckedInternal = _c[1];
    var isCheckedControlled = typeof checkedOriginal !== 'undefined';
    var checked = isCheckedControlled ? checkedMap : checkedInternal;
    var onCheckedChange = react_1.default.useCallback(function (e, name, isChecked) {
        var updatedChecked = tslib_1.__assign({}, checked);
        if (isChecked) {
            updatedChecked[name] = true;
        }
        else {
            delete updatedChecked[name];
        }
        if (!isCheckedControlled) {
            setCheckedInternal(updatedChecked);
        }
        if (onCheckedChangeOriginal) {
            onCheckedChangeOriginal(e, Object.keys(updatedChecked));
        }
    }, [isCheckedControlled, checked, onCheckedChangeOriginal, setCheckedInternal]);
    var selectRadio = react_1.default.useCallback(function (e, name) {
        var updatedChecked = {};
        for (var _i = 0, _a = Object.keys(checked); _i < _a.length; _i++) {
            var checkedName = _a[_i];
            if (!radioItems.includes(checkedName)) {
                // Preserve checked state if non-radio items
                updatedChecked[checkedName] = checked[checkedName];
            }
        }
        updatedChecked[name] = true;
        if (!isCheckedControlled) {
            setCheckedInternal(updatedChecked);
        }
        if (onCheckedChangeOriginal) {
            onCheckedChangeOriginal(e, Object.keys(updatedChecked));
        }
    }, [isCheckedControlled, onCheckedChangeOriginal, setCheckedInternal, checked]);
    // The close arrow key must be handled at this level so that close arrow is responsive when arrowing
    // on a submenu trigger inside a submenu. Otherwise the arrowing event gets "swallowed up" by the trigger,
    // because it is considered to be inside the submenu due to the Menu component wrapping both the
    // trigger and popover. Listening to key events directly here to handle this case doesn't work
    // since left and right arrow key events are already handled and swallowed by native behavior
    var onArrowClose = react_1.default.useCallback(function (e) {
        var _a;
        if (!isSubmenu) {
            return;
        }
        setOpen(e, false /* isOpen */, false /* bubble */);
        (_a = triggerRef === null || triggerRef === void 0 ? void 0 : triggerRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    }, [isSubmenu, setOpen, triggerRef]);
    // Handle 'Home' and 'End' keypresses here. Native menus allow 'Home' and 'End' keypresses to jump to the
    // start and end of each menu list. We need to keep track of all MenuItems in our list, and we need access to
    // (1) the item's ref to focus on and (2) whether the item is disabled or not as different platforms allow
    // and don't allow focus for disabled items.
    // Instead of calling useState to instantiate an array, we call useMemo because we don't want to re-render the
    // MenuList on adding / removing items.
    var trackedMenuItems = react_1.default.useMemo(function () { return []; }, []);
    var trackMenuItem = react_1.default.useCallback(function (item) { return trackedMenuItems.push(item); }, [trackedMenuItems]);
    var untrackMenuItem = react_1.default.useCallback(function (item) {
        return trackedMenuItems.splice(trackedMenuItems.findIndex(function (x) { return x.ref === item.ref; }), 1);
    }, [trackedMenuItems]);
    var onListKeyDown = function (e) {
        var _a;
        var key = e.nativeEvent.key;
        if (handledKeys.includes(key)) {
            // For iOS and macOS, 'Home' and 'End' must set focus on the first and last enabled item.
            // For other platforms, we can just jump to the first and last keys.
            var increment = void 0, idx = void 0;
            if (key === 'Home') {
                increment = 1;
                idx = 0;
            }
            else if (key === 'End') {
                increment = -1;
                idx = trackedMenuItems.length - 1;
            }
            while (platformsWithoutFocusOnDisabled.includes(react_native_1.Platform.OS) && trackedMenuItems[idx].disabled) {
                idx += increment;
            }
            (_a = trackedMenuItems[idx].ref.current) === null || _a === void 0 ? void 0 : _a.focus();
        }
    };
    react_1.default.useEffect(function () {
        return function cleanup() {
            clearTimeout(context.triggerHoverOutTimer);
        };
    });
    // focus management
    var focusZoneRef = react_1.default.useRef();
    var setFocusZoneFocus = function () {
        var _a;
        (_a = focusZoneRef === null || focusZoneRef === void 0 ? void 0 : focusZoneRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    };
    react_1.default.useEffect(function () {
        setFocusZoneFocus();
    }, []);
    return {
        props: tslib_1.__assign(tslib_1.__assign({}, context), { onMouseLeave: handleFocusOnMouseLevae ? setFocusZoneFocus : context.onMouseLeave, onKeyDown: onListKeyDown }),
        isCheckedControlled: isCheckedControlled,
        checked: checked,
        onArrowClose: onArrowClose,
        onCheckedChange: onCheckedChange,
        selectRadio: selectRadio,
        addRadioItem: addRadioItem,
        removeRadioItem: removeRadioItem,
        trackMenuItem: trackMenuItem,
        untrackMenuItem: untrackMenuItem,
        hasMaxHeight: (_a = context.hasMaxHeight) !== null && _a !== void 0 ? _a : false,
        hasMaxWidth: (_b = context.hasMaxWidth) !== null && _b !== void 0 ? _b : false,
        focusZoneRef: focusZoneRef,
    };
};
exports.useMenuList = useMenuList;
// Hook called in individual MenuItems to keep track of their refs - this is for "Home" and "End" functionality.
var useMenuItemTracking = function (ref, disabled) {
    var _a = (0, menuListContext_1.useMenuListContext)(), trackMenuItem = _a.trackMenuItem, untrackMenuItem = _a.untrackMenuItem;
    var item = react_1.default.useMemo(function () { return ({
        ref: ref,
        disabled: disabled,
    }); }, [ref, disabled]);
    // We only want to call this once - when the item initially renders.
    react_1.default.useEffect(function () {
        trackMenuItem(item);
        return function () { return untrackMenuItem(item); };
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
};
exports.useMenuItemTracking = useMenuItemTracking;
//# sourceMappingURL=useMenuList.js.map